@@ -6,7 +6,7 @@
 :License: MIT
 """
 
-from ..xml.utils import validate_xpaths_ref_to_unique_objects
+from ..xml.utils import validate_xpaths_ref_to_unique_objects, eval_xpath
 from .data_model import (SedIdGroupMixin, AbstractTask, Task, RepeatedTask,  # noqa: F401
                          Model, ModelLanguage, ModelLanguagePattern,
                          ModelChange, ComputeModelChange,
@@ -16,9 +16,12 @@ from .data_model import (SedIdGroupMixin, AbstractTask, Task, RepeatedTask,  # n
                          SetValueComputeModelChange,
                          Report, Plot2D, Plot3D, DataGenerator,
                          Calculation)
+from .math import compile_math, eval_math
 from .utils import (append_all_nested_children_to_doc, get_range_len,
                     is_model_language_encoded_in_xml, get_models_referenced_by_task,
-                    compile_math, eval_math, get_all_sed_objects)
+                    get_all_sed_objects,
+                    get_data_generators_for_output, get_variables_for_data_generators,
+                    get_model_changes_for_task)
 import collections
 import copy
 import lxml.etree
@@ -41,6 +44,7 @@ __all__ = [
     'validate_simulation',
     'validate_uniform_range',
     'validate_output',
+    'validate_data_generator',
     'validate_data_generator_variables',
     'validate_target',
     'validate_variable_xpaths',
@@ -262,7 +266,7 @@ def validate_doc(doc, working_dir, validate_semantics=True, validate_models_with
 
                             if variable.target and variable.model and variable.model.language:
                                 temp_errors, temp_warnings = validate_target(
-                                    variable.target, variable.target_namespaces, Model, variable.model.language)
+                                    variable.target, variable.target_namespaces, Calculation, variable.model.language, variable.model.id)
                                 variable_errors.extend(temp_errors)
                                 variable_warnings.extend(temp_warnings)
 
@@ -404,7 +408,7 @@ def validate_doc(doc, working_dir, validate_semantics=True, validate_models_with
                     if change.target:
                         if change.model and change.model.language:
                             temp_errors, temp_warnings = validate_target(
-                                change.target, change.target_namespaces, Model, change.model.language)
+                                change.target, change.target_namespaces, ModelChange, change.model.language, change.model.id)
                             change_errors.extend(temp_errors)
                             change_warnings.extend(temp_warnings)
 
@@ -441,7 +445,7 @@ def validate_doc(doc, working_dir, validate_semantics=True, validate_models_with
 
                         if variable.target and variable.model and variable.model.language:
                             temp_errors, temp_warnings = validate_target(
-                                variable.target, variable.target_namespaces, Model, variable.model.language)
+                                variable.target, variable.target_namespaces, Calculation, variable.model.language, variable.model.id)
                             variable_errors.extend(temp_errors)
                             variable_warnings.extend(temp_warnings)
 
@@ -479,52 +483,55 @@ def validate_doc(doc, working_dir, validate_semantics=True, validate_models_with
 
             if task_errors[task]['other']:
                 errors.append(['Task {} is invalid.'.format(task_id), task_errors[task]['other']])
+
             if task_warnings[task]['other']:
                 warnings.append(['Task {} may be invalid.'.format(task_id), task_warnings[task]['other']])
 
-        # variables of data generators
-        # - have ids
-        # - have target OR symbol
-        # - don't have model references
-        # - have math
+        # validate data generators
+        model_etrees = {}
+        for model in doc.models:
+            if (
+                model.language
+                and is_model_language_encoded_in_xml(model.language)
+                and model.source
+                and not model.source.startswith('#')
+                and not model.source.startswith('http://')
+                and not model.source.startswith('https://')
+            ):
+                if os.path.isabs(model.source):
+                    model_source = model.source
+                else:
+                    model_source = os.path.join(working_dir, model.source)
+
+                if os.path.isfile(model_source):
+                    try:
+                        model_etrees[model] = lxml.etree.parse(model_source)
+                    except Exception:
+                        pass
+
         for i_data_gen, data_gen in enumerate(doc.data_generators):
-            data_gen_errors = []
-            data_gen_warnings = []
+            data_gen_errors, data_gen_warnings = validate_data_generator(data_gen, model_etrees=model_etrees)
 
-            for i_parameter, param in enumerate(data_gen.parameters):
-                if not param.id:
-                    data_gen_errors.append(['Parameter {} must have an id.'.format(i_parameter + 1)])
-
-            temp_errors, temp_warnings = validate_data_generator_variables(data_gen.variables)
-            data_gen_errors.extend(temp_errors)
-            data_gen_warnings.extend(temp_warnings)
-
-            temp_errors, temp_warnings = validate_calculation(data_gen)
-            data_gen_errors.extend(temp_errors)
-            data_gen_warnings.extend(temp_warnings)
+            data_gen_id = '`' + data_gen.id + '`' if data_gen and data_gen.id else str(i_data_gen + 1)
 
             if data_gen_errors:
-                data_gen_id = '`' + data_gen.id + '`' if data_gen and data_gen.id else str(i_data_gen + 1)
                 errors.append(['Data generator {} is invalid.'.format(data_gen_id), data_gen_errors])
 
             if data_gen_warnings:
-                data_gen_id = '`' + data_gen.id + '`' if data_gen and data_gen.id else str(i_data_gen + 1)
                 warnings.append(['Data generator {} may be invalid.'.format(data_gen_id), data_gen_warnings])
 
         # validate outputs
-        # - reports
-        #   - data sets have ids and labels
-        #   - data sets reference data generators
-        # - plots
-        #   - curves and surfaces have ids
-        #   - x, y, z attributes reference data generators
         for i_output, output in enumerate(doc.outputs):
-            output_errors = validate_output(output)
+            output_errors, output_warnings = validate_output(output)
+
+            output_id = '`' + output.id + '`' if output and output.id else str(i_output + 1)
 
             if output_errors:
-                output_id = '`' + output.id + '`' if output and output.id else str(i_output + 1)
                 errors.append(['Output {} is invalid.'.format(output_id), output_errors])
 
+            if output_warnings:
+                warnings.append(['Output {} may be invalid.'.format(output_id), output_warnings])
+
     return (errors, warnings)
 
 
@@ -766,7 +773,7 @@ def validate_model_changes(model):
 
         if change.target:
             if model.language:
-                temp_errors, temp_warnings = validate_target(change.target, change.target_namespaces, Model, model.language)
+                temp_errors, temp_warnings = validate_target(change.target, change.target_namespaces, ModelChange, model.language, model.id)
                 change_errors.extend(temp_errors)
                 change_warnings.extend(temp_warnings)
 
@@ -797,7 +804,7 @@ def validate_model_changes(model):
 
                 if variable.target and variable.model and variable.model.language:
                     temp_errors, temp_warnings = validate_target(
-                        variable.target, variable.target_namespaces, Model, variable.model.language)
+                        variable.target, variable.target_namespaces, Calculation, variable.model.language, variable.model.id)
                     variable_errors.extend(temp_errors)
                     variable_warnings.extend(temp_warnings)
 
@@ -807,7 +814,7 @@ def validate_model_changes(model):
 
                 if variable_warnings:
                     var_id = '`' + variable.id + '`' if variable and variable.id else str(i_variable + 1)
-                    change_warnings.append(['Variable {} is invalid.'.format(var_id), variable_warnings])
+                    change_warnings.append(['Variable {} may be invalid.'.format(var_id), variable_warnings])
 
             temp_errors, temp_warnings = validate_calculation(change)
             change_errors.extend(temp_errors)
@@ -932,15 +939,56 @@ def validate_uniform_range(range):
     return errors
 
 
-def validate_data_generator_variables(variables):
+def validate_data_generator(data_generator, model_etrees=None):
+    """ Validate a data generator
+
+    * Parameters have ids
+    * Variables have ids
+    * Variables have target OR symbol
+    * Variables don't have model references
+    * Have math
+    * Math is valid
+
+    Args:
+        data_generator (:obj:`DataGenerator`): data generator
+        model_etrees (:obj:`dict`, optional): dictionary that maps models to XML element trees of their sources
+
+    Returns:
+        :obj:`tuple`:
+
+            * nested :obj:`list` of :obj:`str`: nested list of errors (e.g., required ids missing or ids not unique)
+            * nested :obj:`list` of :obj:`str`: nested list of warnings (e.g., required ids missing or ids not unique)
+    """
+    errors = []
+    warnings = []
+
+    for i_parameter, param in enumerate(data_generator.parameters):
+        if not param.id:
+            errors.append(['Parameter {} must have an id.'.format(i_parameter + 1)])
+
+    temp_errors, temp_warnings = validate_data_generator_variables(data_generator.variables, model_etrees=model_etrees)
+    errors.extend(temp_errors)
+    warnings.extend(temp_warnings)
+
+    temp_errors, temp_warnings = validate_calculation(data_generator)
+    errors.extend(temp_errors)
+    warnings.extend(temp_warnings)
+
+    return (errors, warnings)
+
+
+def validate_data_generator_variables(variables, model_etrees=None):
     """ Check variables have a symbol or target
 
     Args:
         variables (:obj:`list` of :obj:`Variable`): variables
+        model_etrees (:obj:`dict`, optional): dictionary that maps models to XML element trees of their sources
 
     Returns:
         nested :obj:`list` of :obj:`str`: nested list of errors (e.g., required ids missing or ids not unique)
     """
+    model_etrees = model_etrees or {}
+
     errors = []
     warnings = []
 
@@ -964,7 +1012,14 @@ def validate_data_generator_variables(variables):
             models = get_models_referenced_by_task(variable.task)
             for model in models:
                 if model and model.language:
-                    temp_errors, temp_warnings = validate_target(variable.target, variable.target_namespaces, DataGenerator, model.language)
+                    model_changes = model.changes or list(filter(lambda change: change.model == model,
+                                                                 get_model_changes_for_task(variable.task)))
+
+                    temp_errors, temp_warnings = validate_target(variable.target, variable.target_namespaces,
+                                                                 DataGenerator, model.language,
+                                                                 model_id=model.id,
+                                                                 model_etree=model_etrees.get(model, None),
+                                                                 check_in_model_source=not model_changes)
                     variable_errors.extend(temp_errors)
                     variable_warnings.extend(temp_warnings)
 
@@ -974,7 +1029,7 @@ def validate_data_generator_variables(variables):
 
         if variable_warnings:
             variable_id = '`' + variable.id + '`' if variable and variable.id else str(i_variable + 1)
-            warnings.append(['Variable {} is invalid.'.format(variable_id), variable_warnings])
+            warnings.append(['Variable {} may be invalid.'.format(variable_id), variable_warnings])
 
     return errors, warnings
 
@@ -982,13 +1037,26 @@ def validate_data_generator_variables(variables):
 def validate_output(output):
     """ Validate an output
 
+    * Reports
+      * Data sets have ids and labels
+      * Data sets reference data generators
+
+    * Plots
+
+       * Curves and surfaces have ids
+       * x, y, z attributes reference data generators
+
     Args:
         output (:obj:`Output`): output
 
     Returns:
-        nested :obj:`list` of :obj:`str`: nested list of errors (e.g., required ids missing or ids not unique)
+        :obj:`tuple`:
+
+            * nested :obj:`list` of :obj:`str`: nested list of errors (e.g., required ids missing or ids not unique)
+            * nested :obj:`list` of :obj:`str`: nested list of warnings (e.g., required ids missing or ids not unique)
     """
     errors = []
+    warnings = []
 
     if isinstance(output, Report):
         for i_data_set, data_set in enumerate(output.data_sets):
@@ -1033,10 +1101,22 @@ def validate_output(output):
                 surface_id = '`' + surface.id + '`' if surface and surface.id else str(i_surface + 1)
                 errors.append(['Surface {} is invalid.'.format(surface_id), surface_errors])
 
-    return errors
+    involves_repeated_task = False
+    for variable in get_variables_for_data_generators(get_data_generators_for_output(output)):
+        if variable.task and not isinstance(variable.task, Task):
+            involves_repeated_task = True
+
+    if involves_repeated_task:
+        msg = (
+            'Some simulation tools will not be able to generate this output because it uses data from repeated tasks. '
+            'Output for repeated tasks is an experimental feature of SED-ML, which is not officially supported.'
+        )
+        warnings.append([msg])
+
+    return (errors, warnings)
 
 
-def validate_target(target, namespaces, context, language, doc=None):
+def validate_target(target, namespaces, context, language, model_id, model_etree=None, doc=None, check_in_model_source=False):
     """ Validate that a target is a valid XPath and that the namespaces needed to resolve a target are defined
 
     Args:
@@ -1044,7 +1124,10 @@ def validate_target(target, namespaces, context, language, doc=None):
         namespaces (:obj:`dict`): dictionary that maps prefixes of namespaces to their URIs
         context (:obj:`type`)
         language (:obj:`str`): model language
-        doc (:obj:`SedDocument`): SED document
+        model_id (:obj:`str`): model id
+        model_etree (:obj:`etree.Element`, optional): XML element tree for model source
+        doc (:obj:`SedDocument`, optional): SED document
+        check_in_model_source (:obj:`bool`, optional): whether to check that the target exists in the source
 
     Returns:
         nested :obj:`list` of :obj:`str`: nested list of errors (e.g., required ids missing or ids not unique)
@@ -1079,6 +1162,20 @@ def validate_target(target, namespaces, context, language, doc=None):
             root = lxml.etree.Element("root")
             try:
                 xpath.evaluate(root)
+
+                if model_etree and check_in_model_source and context in [DataGenerator, Calculation]:
+                    if context == DataGenerator and '/@' in target:
+                        xpath = target.rpartition('/@')[0]
+                    else:
+                        xpath = target
+
+                    objs = eval_xpath(model_etree, xpath, namespaces)
+
+                    if not objs:
+                        errors.append(['XPath `{}` does not match any elements of model `{}`.'.format(xpath, model_id or '')])
+                    elif len(objs) > 1:
+                        errors.append(['XPath `{}` matches multiple elements of model `{}`.'.format(xpath, model_id or '')])
+
             except lxml.etree.XPathEvalError as exception:
                 if 'Undefined namespace prefix' in str(exception):
                     if namespaces:
